import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:json_annotation/json_annotation.dart';

part 'blossom_tab.g.dart';

T _dataFromJson<T>(
        Map<String, dynamic> data, T Function(Map<String, dynamic>) dataFromJson) =>
    data.keys.length == 1 && data.keys.first == 'value'
        ? data['value']
        : dataFromJson(data);

Map<String, dynamic>? _dataToJson<T>(T data) =>
    data is String || data is int || data is double || data is bool
        ? {'value': data}
        : (data as dynamic)?.toJson();

@JsonSerializable()
class BlossomTab<T> {
  BlossomTab({
    required this.id,
    this.data,
    this.title,
    this.isSticky = false,
    this.maxSize = 200,
    this.stickySize = 50,
  });
  final String id;
  @JsonKey(fromJson: _dataFromJson, toJson: _dataToJson)
  final T? data;
  final String? title;
  final bool isSticky;
  final double maxSize;
  final double stickySize;

  static BlossomTab<S> fromJson<S>(
          Map<String, dynamic> json, S Function(Map<String, dynamic>) dataFromJson) =>
      _$BlossomTabFromJson<S>(json, dataFromJson);
  Map<String, dynamic> toJson() => _$BlossomTabToJson(this);
}
